#!/bin/csh
# usage : bar_[l,g]s.csh width height size left right {BPS bar.tif}
#
#set tmp=/tmp/bar.$$
set tmp=/dev/shm/bar.$$
#
alias STDERR 'echo echo 1\>\&2 "\!*" | /bin/sh'
#
set csh=`basename $0`
switch($csh)
  case bar_ls.csh :
    set bar=(0 setgray rectfill) ; breaksw
  case bar_gs.csh :
    set bar=(4 2 roll translate scale 256 1 8 \[ 256 0 0 1 0 0 \] \
             256 string 0 1 255 \{ 1 index exch dup put \} for image \
             0 setgray 0 setlinewidth 0 0 1 1 rectstroke) ; breaksw
  default :
    STDERR $csh : unknown C-shell script. ; exit 1
endsw
#
if ($#argv != 5 && $#argv != 7) then
  STDERR usage : $csh width height size left right \{BPS bar.tif\} ; exit 1
endif
#
set bw=`echo "$argv[1]" | bc -l | awk '$1>0 { print }'`
if ($#bw != 1) then
  STDERR $argv[1] : bad bar width. ; exit 1
endif
#
set bh=`echo "$argv[2]" | bc -l | awk -v BW=$bw \
                                      '$1!=0 { print ($1<0)?-BW*$1:$1 }'`
if ($#bh != 1) then
  STDERR $argv[2] : bad bar height. ; exit 1
endif
#
set fs=`echo "$argv[3]" | bc -l | awk -v BW=$bw \
                                      '$1!=0 { print ($1<0)?-BW*$1:$1 }'`
if ($#fs != 1) then
  STDERR $argv[3] : bad font size. ; exit 1
endif
#
set y=(`echo $bh $fs | awk '{ if ($1<$2) { f=0; b=($2-$1)/2; y=$2 } else \
                                         { f=($1-$2)/2; b=0; y=$1 } \
                              print 0.224*$2+f+1,b+1,y+2,int(y+2) }'`)
#
cat <<fin >$tmp
/Helvetica findfont $fs scalefont setfont
1 $y[1] moveto
(${argv[4]}) dup show
stringwidth pop 1 add dup
$bw add $fs add $y[1] moveto
(${argv[5]}) show
$fs 2 div add $y[2] $bw $bh
$bar
showpage
fin
#
set x=(`gs -q -sDEVICE=bbox -dNOPAUSE -dBATCH $tmp |& \
        awk '$1=="%%HiResBoundingBox:" { print $2-1,$4+1,int($4-$2+2) }'`)
#
if ($#argv == 5) then
  echo %\!PS-Adobe-2.0
  echo %%BoundingBox: $x[1] 0 $x[2] $y[3]
  echo %%PageBoundingBox: $x[1] 0 $x[2] $y[3]
  cat $tmp
else
  ( echo $x[1] neg 0 translate ; cat $tmp ) | \
  gs -q -sDEVICE=tiffgray -r72 -g${x[3]}x$y[4] -sOutputFile=$argv[7] -
  echo 0 255 0 `echo 2\^$argv[6] -1 | bc` | t_pvr $argv[7] $argv[7] >/dev/null
endif
#
rm $tmp
