#!/bin/csh
# usage : draw_[a,p].csh word|byte x y z Dr/Dz {HG_size_inch}
#
set hgs=3	# histogram size in inch
#
set fs=9	# font size in pt.
set tl=6	# tick length in pt.
#
set cfx=(1 1 1)	# color of font for x-slice
set ctx=(1 0 0)	# color of tick for x-slice
set cfy=(1 1 1)	# color of font for y-slice
set cty=(0 1 0)	# color of tick for y-slice
set cfz=(1 1 1)	# color of font for z-slice
set ctz=(0 0 1)	# color of tick for z-slice
#
set sih=18	# space between images and histogram in pt.
#
#set tmp=/tmp/draw.$$
set tmp=/dev/shm/draw.$$
#
set csh=`basename $0`
switch($csh)
  case draw_a.csh :
    set plot=plot_a.csh ; breaksw
  case draw_p.csh :
    set plot=plot_p.csh ; breaksw
  default :
    stderr $csh : unknown C-shell script. ; exit 1
endsw
#
if ($#argv != 5 && $#argv != 6) then
  stderr usage : $csh word\|byte x y z Dr/Dz \{HG_size_inch\} ; exit 1
endif
#
set w_b=$argv[1]
if (!(-d $w_b)) then
  stderr $w_b : directory not found. ; exit 1
endif
#
set csv=${w_b}.csv
if (!(-e $csv)) then
  stderr $csv : CSV file not found. ; exit 1
endif
#
set xyz=(`ls $w_b | \
          awk -v D=$w_b -v x=$argv[2] -v y=$argv[3] -v z=$argv[4] \
              '{ (p="pig " (f=D "/" $0)) | getline; close(p); \
                 if (NF<4) exit 1; \
                 if ($3!=1 || $4>16) { \
                   print f,": bad image type." > "/dev/stderr"; exit 1 } \
                 if (X<$1) X=$1; \
                 if (Y<$2) Y=$2; ++Z } \
               END { if (x<0 || x>=X) print x; else \
                     if (y<0 || y>=Y) print y; else \
                     if (z<0 || z>=Z) print z; else print X,Y,Z,z+1 }'`)
if ($#xyz == 0) exit 1
if ($#xyz == 1) then
  stderr $xyz : bad position on image. ; exit 1
endif
#
set dpi=(`echo "t= $xyz[3] / $hgs ; t/( $argv[5] ) ; t" | bc -l`)
if ($#dpi != 2) then
  stderr "$argv[5]" : bad ratio of Dr / Dz. ; exit 1
endif
#
if ($#argv == 5) then
  set o=($sih $sih)
else
  set o=(`( echo -n "$sih $hgs " ; echo "$argv[6]" | bc -l ) | \
          awk 'NF==3 && $3>0 { o=(m=$3/$2)*$1; print o,o,"-m",m,m }'`)
  if ($#o == 0) then
    stderr "$argv[6]" : bad histogram size in inch. ; exit 1
  endif
  set fs=`echo $fs / $o[$#o] | bc -l`
endif
#
( echo +x +z +y $argv[3] ${tmp}.xz ; \
  echo +z +y +x $argv[2] ${tmp}.zy ) | si_rar $w_b -
#
( echo /Helvetica findfont $fs scalefont setfont ; \
  echo /S \{ setrgbcolor \
             $fs 1.375 mul sub exch \
             $fs 0.5 mul sub 2 index stringwidth pop sub exch \
             newpath moveto show \} def ; \
  echo /H \{ setrgbcolor 72 exch div setlinewidth \
             $tl neg exch newpath moveto $tl 0 rlineto \
             0 rmoveto $tl 0 rlineto stroke \} def ; \
  echo /V \{ setrgbcolor 72 exch div setlinewidth \
             $tl neg newpath moveto 0 $tl rlineto \
             0 exch rmoveto 0 $tl rlineto stroke \} def ; \
  echo /x $argv[2] 0.5 add $dpi[1] div 72 mul def ; \
  echo /y $argv[3] 0.5 add $dpi[1] div 72 mul def ; \
  echo /z $argv[4] 0.5 add $dpi[2] div 72 mul def ; \
  echo /X $xyz[1] $dpi[1] div 72 mul def ; \
  echo /Y $xyz[2] $dpi[1] div 72 mul def ; \
  echo /Z $hgs 72 mul def ; \
  ( t2ps_P `ls $w_b | sed -n ${xyz[4]}s@\^@${w_b}/@gp` $dpi[1] - - ; \
    echo \(z = ${argv[4]}\) X Y $cfz S X Y y sub $dpi[1] $cty H \
                                       Y x $dpi[1] $ctx V showpage ; \
    t2ps_P ${tmp}.xz $dpi[1] $dpi[2] - - ; \
    echo \(y = ${argv[3]}\) X Z $cfy S X Z z sub $dpi[2] $ctz H \
                                       Z x $dpi[1] $ctx V showpage ; \
    t2ps_P ${tmp}.zy $dpi[2] $dpi[1] - - ; \
    echo \(x = ${argv[2]}\) Z Y $cfx S Z Y y sub $dpi[1] $cty H \
                                       Y z $dpi[2] $ctz V showpage ) | \
  sed "s/^%%%/0 setlinewidth /g" ; \
  $plot $csv $hgs ) | \
sed s/\^%%BoundingBox:/%%PageBoundingBox:/g | epspile -o $o -n 2 2
#
rm ${tmp}.*
