
#include <stdio.h>
#include <stdlib.h>

static void	Error(char *msg)
{
	(void)fputs(msg,stderr); (void)fputc('\n',stderr); exit(1);
}

static unsigned	Image=0;

#define LEN	2048

static void	Abort(char *msg)
{
	char	str[LEN];

	(void)sprintf(str,"%s at image %u.",msg,Image); Error(str);
}

static void	Skip(FILE *file,size_t size,char *msg)
{
	char	str[LEN];

	if (file!=stdin) {
	    if (fseeko(file,(off_t)size,SEEK_CUR)) goto abort;
	}
	else
	    do if (fgetc(file)==EOF) goto abort; while (--size!=0);

	return;
abort:;
	(void)sprintf(str,"%s skip error",msg); Abort(str);
}

#define SHORT	unsigned short

static unsigned	Np,Nz,Npz,NumOfImages=0;
static SHORT	RemChannels=1;
static size_t	SNpz;

static void	Header(FILE *file,char *msg)
{
	char		str[LEN],header[64];
	unsigned	*NOI=(unsigned *)(header+14);
	SHORT		*CL=(SHORT *)(header+2),
			*IW=(SHORT *)(header+4),
			*IH=(SHORT *)(header+6),
			*FT=(SHORT *)(header+12),
			*RC=(SHORT *)(header+18);

	if (feof(file) || (NumOfImages==0 && RemChannels==0)) {
	    (void)sprintf(str,"missing %s",msg); Abort(str);
	}
	if (fread(&header,64,1,file)!=1 ||
	    header[0]!='I' || header[1]!='M') Abort("header read error");

	if (*FT!=2) Abort("unsupported file type");

	if (Image==0) {
	    if (*IW==0 || *IH==0) Abort("bad size");

	    SNpz=sizeof(SHORT)*(Npz=(Np=(*IW))*(Nz=(*IH)));
	}
	else
	    if (*IW!=Np || *IH!=Nz) Abort("size not match");

	if (*CL!=0) Skip(file,*CL,"comment");

	if (NumOfImages==0) {
	    NumOfImages=(*NOI); RemChannels=(*RC);
	}
	--NumOfImages;
}

static void	Read(FILE *file,SHORT *data,char *msg)
{
	char	str[LEN];

	if (fread(data,SNpz,1,file)!=1) {
	    (void)sprintf(str,"%s read error",msg); Abort(str);
	}
}

#define ULL	unsigned long long

int	main(int argc,char **argv)
{
	FILE		*HIS;
	int		Ns,Nv,Nd,Nh,z1,z2,h,v,s,z,p,p0,p1,p2,pw=(-1),mm=1;
	double		min,max,base,step,*D,id,ph;
	ULL		*H;
	SHORT		*I;
	unsigned	pz;

	if (argc!=8 && argc!=9 && argc!=10 && argc!=11)
    Error("usage : his2hg HIS scans views darks {z1 z2} {width} min max bins");

	if (argv[1][0]=='-')
	    HIS=stdin;
	else
	    if ((HIS=fopen(argv[1],"rb"))==NULL) Error("file not found.");

	if ((Ns=atoi(argv[2]))<2) Error("bad number of scans.");
	if ((Nv=atoi(argv[3]))<1) Error("bad number of views.");
	if ((Nd=atoi(argv[4]))<1) Error("bad number of darks.");

	if ((argc==9 || argc==11) &&
	    (pw=atoi(argv[argc-4]))<0) Error("bad peak width.");

	if ((min=atof(argv[argc-3]))>=
	    (max=atof(argv[argc-2])) ||
	    (Nh=atoi(argv[argc-1]))<=0) Error("bad histogram geometry.");

	base=min; step=(max-min)/(double)Nh;

	for (Image=0; Image<Nd; Image++) {
	    Header(HIS,"dark data");

	    if (Image==0) {
		if (argc==8 || argc==9) {
		    z1=0; z2=Nz-1;
		}
		else
		    if ((z1=atoi(argv[5]))<0 ||
			(z2=atoi(argv[6]))>=Nz || z1>z2)
			Error("bad slice range.");

		if ((D=(double *)malloc(sizeof(double)*Npz))==NULL ||
		    (H=(ULL *)malloc(sizeof(ULL)*(Nh+2)))==NULL ||
		    (I=(SHORT *)malloc(SNpz))==NULL)
		    Error("memory allocation error.");

		for (pz=0; pz<Npz; pz++) D[pz]=0.0;

		++H; for (h=(-1); h<=Nh; h++) H[h]=0;
	    }
	    Read(HIS,I,"dark data");

	    for (pz=0; pz<Npz; pz++) D[pz]+=(double)I[pz];
	}
	for (pz=0; pz<Npz; pz++) D[pz]/=(double)Nd;

	for (v=0; v<Nv; v++) {
	    Header(HIS,"junk data"); Skip(HIS,SNpz,"junk data"); ++Image;

	    for (s=1; s<Ns; s++) {
		Header(HIS,"data"); Read(HIS,I,"data"); ++Image;

		for (z=z1; z<=z2; z++) {
		    for (pz=Np*z, p=0; p<Np; p++, pz++) {
			id=(double)I[pz]-D[pz];

			if (p==0 || id>ph) {
			    p0=p; ph=id;
			}
		    }
		    p1=p0-pw; p2=p0+pw;
		    for (pz=Np*z, p=0; p<Np; p++, pz++) if (p<p1 || p>p2) {
			id=(double)I[pz]-D[pz];

			if (mm) {
			    mm=0; min=max=id;
			}
			else
			    if (id<min) min=id; else if (id>max) max=id;

			++H[(id<base)?-1:((h=(int)((id-base)/step))>=Nh)?Nh:h];
		    }
		}
	    }
	}
	if (HIS!=stdin) (void)fclose(HIS);

	(void)fprintf(stderr,"%lf\t%lf\n",min,max);

	for (h=(-1); h<=Nh; h++) if (H[h]!=0)
	    (void)printf("%d\t%lf\t%lf\t%llu\n",
			 h,base+step*(double)h,base+step*(double)(h+1),H[h]);

	free(I); free(H-1); free(D);

	return 0;
}
