
#include <stdio.h>
#include <stdlib.h>
#include "cell.h"
#include "sif.h"

extern void	Error(char *msg);

typedef struct RL {
			unsigned	i1,i2;
			struct RL	*next;
	} RL;

#define LIM	(unsigned)~0

static RL	*SetUp(char *arg)
{
	int	c;
	size_t	i1,i2;
	RL	*next,*rl=NULL;
	char	*brl="bad range list.";

#define GET(c)	(c=(*(arg++)))
#define S2I(i)	\
	for (i=0; c>='0' && c<='9'; GET(c)) if ((i=i*10+c-'0')>LIM) Error(brl)

	do {
	    while (GET(c)==',') ;

	    if (c!='\0') {
		if (c=='-') {
		    i1=0;
		    if (GET(c)==',' || c=='\0')
			i2=LIM;
		    else {
			S2I(i2);
			if (c!=',' && c!='\0') Error(brl);
		    }
		}
		else {
		    S2I(i1);
		    if (c==',' || c=='\0')
			i2=i1;
		    else if (c=='-')
			if (GET(c)==',' || c=='\0')
			    i2=LIM;
			else {
			    S2I(i2);
			    if ((c!=',' && c!='\0') || i2<i1) Error(brl);
			}
		    else
			Error(brl);
		}
		next=rl;
		if ((rl=(RL *)malloc(sizeof(RL)))==NULL)
		    Error("no allocatable memory for range list.");

		rl->i1=(unsigned)i1; rl->i2=(unsigned)i2; rl->next=next;
	    }
	} while (c==',');

	return rl;
}

static int	Include(RL *rl,unsigned i)
{
	int	e=2;

	for (; rl!=NULL; rl=rl->next)
	    if (i<=rl->i2) if (i>=rl->i1) return 1; else e=0;

	return e;
}

#define LEN	2048

static void	Abort(char *msg,unsigned i)
{
	char	str[LEN];

	(void)sprintf(str,"%s at image %u.",msg,i); Error(str);
}

#define ULL	unsigned long long

int	main(int argc,char **argv)
{
	FILE		*file;
	RL		*rl,*next;
	char		H[64],*desc,ts[LEN],path[LEN];
	unsigned	i,bits,u,
			*NI=(unsigned *)(H+14),	/* number of images */
			ni=0;
	short		*CL=(short *)(H+2),	/* comment length */
			*IW=(short *)(H+4),	/* image width */
			*IH=(short *)(H+6),	/* image height */
			*FT=(short *)(H+12),	/* file type */
			*RC=(short *)(H+18),	/* remaining channels */
			rc=1;
	double		*TS=(double *)(H+22);	/* time stamp */
	Cell		**cell;

	if (argc!=4) Error("usage : his2tiff HIS_file rangeList TIFF_format");

	if (argv[1][0]=='-')
	    file=stdin;
	else
	    if ((file=fopen(argv[1],"rb"))==NULL) Error("file not found.");

	if ((rl=SetUp(argv[2]))==NULL) Error("empty range list.");

	for (i=0; !feof(file) && (ni!=0 || rc!=0); i++) {
	    if (fread(&H,64,1,file)!=1 ||
		H[0]!='I' || H[1]!='M') Abort("header read error",i);

	    switch(*FT) {
		case  1: bits= 8; break; case 11: bits=24; break;
		case  2: bits=16; break; case 12: bits=48; break;
		case  3: bits=32; break; case 13: bits=96; break;
		case  6: bits=12; break; case 14: bits=36; break;
		default: Abort("unknown file type",i);
	    }

#define SIZE	(unsigned)(((ULL)*IW*(*IH)*bits+7)/8)

	    switch(Include(rl,i)) {
		case 0:
		    if (file!=stdin)
			(void)fseeko(file,(off_t)(*CL+SIZE),SEEK_CUR);
		    else
			for (u=(*CL)+SIZE; u>0; u--) (void)fgetc(file);

		    break;
		case 1:
		    if (bits!=sizeof(Cell)*8) Abort("bad bits per sample",i);

		    if ((*CL!=0 && ((desc=(char *)malloc(*CL+1))==NULL ||
				    fread(desc,*CL,1,file)!=1)) ||
			(cell=(Cell **)malloc(sizeof(Cell *)*(*IH)))==NULL ||
			(*cell=(Cell *)malloc(SIZE))==NULL ||
			fread(*cell,SIZE,1,file)!=1)
			Abort("memory allocation or data read error",i); 

		    if (*CL!=0)
			desc[*CL]='\0';
		    else
			desc=(sprintf(ts,"%lf",*TS)<0)?NULL:ts;

		    for (u=1; u<(*IH); u++) cell[u]=cell[u-1]+(*IW);

		    (void)sprintf(path,argv[3],i);
		    StoreImageFile(path,(int)*IW,(int)*IH,(int)bits,cell,desc);

		    free(*cell); free(cell); if (*CL!=0) free(desc);

		    break;
		default:
		    goto Break;
	    }
	    if (ni==0) {
		ni=(*NI); rc=(*RC);
	    }
	    --ni;
	}
Break:;
	for (; rl!=NULL; rl=next) {
	    next=rl->next; free(rl);
	}
	if (file!=stdin) (void)fclose(file);

	return 0;
}
