
#include <stdio.h>
#include <stdlib.h>
#include "srf.h"
#include "sif_r.h"

#ifdef	XT
#define LOG(I,T)	T/I
#else
#include <math.h>

#define LOG(I,T)	log(I/T)
#endif

extern void	Error(char *msg);

static void	Abort(char *head,char *tail)
{
	(void)fputs(head,stderr); (void)fputs(" : ",stderr); Error(tail);
}

static int	GetW(FILE *file)
{
	int	lo,hi;

	return ((lo=fgetc(file))==EOF || (hi=fgetc(file))==EOF)?-1:hi*256+lo;
}

static FILE	*Open(char *path,int *Nx,int *Ny)
{
	FILE	*file;
	int	cl,ny;

	if ((file=fopen(path,"rb"))==NULL) Abort(path,"file not found.");

	if (fgetc(file)!='I' ||
	    fgetc(file)!='M') Abort(path,"bad magic number.");

	if ((cl=GetW(file))<0) Abort(path,"bad comment length.");

	if ((*Nx=GetW(file))<0 ||
	    ( ny=GetW(file))<0) Abort(path,"bad image size.");

	(void)fseek(file,4L,SEEK_CUR);

	if (GetW(file)!=2) Abort(path,"bad image type.");

	(void)fseek(file,50L+cl,SEEK_CUR);

	if (Ny!=NULL) *Ny=ny;

	return file;
}

typedef unsigned short	Word;

static void	Read(char *path,int x1,int y1,int x2,int y2,Word *word)
{
	int	Nx,y,x,w,nx=x2-x1+1;
	FILE	*file=Open(path,&Nx,NULL);
	short	s=127;
	char	*c=(char *)&s,*dre="data read error.";
	long	o1=2L*x1,o2=2L*(Nx-1-x2);

	(void)fseek(file,2L*Nx*y1,SEEK_CUR);

	if (*c==127)
	    for (y=y1; y<=y2; y++, word+=nx) {
		(void)fseek(file,o1,SEEK_CUR);

		if (fread(word,2L,nx,file)!=nx) Abort(path,dre);

		(void)fseek(file,o2,SEEK_CUR);
	    }
	else
            for (y=y1; y<=y2; y++) {
                (void)fseek(file,o1,SEEK_CUR);

                for (x=x1; x<=x2; x++, word++)
                    if ((w=GetW(file))>=0) *word=(Word)w; else Abort(path,dre);

                (void)fseek(file,o2,SEEK_CUR);
            }

        (void)fclose(file);
}

typedef struct {
		int	n;
		double	c,a;
		int	t;
	} Log;

static int	Compare(Log *l1,Log *l2)
{
	int	t=l1->t-l2->t;

	return (t)?t:(l1->t<0)?(l1->c>l2->c)?1:(l1->c<l2->c)?-1:0:
		     (l1->t>0)?(l1->a>l2->a)?1:(l1->a<l2->a)?-1:0:0;
}

#define LEN	2048

int	main(int argc,char **argv)
{
	char	*O,*D,**Q,S[LEN];
	int	N,Nx,Ny,n,t,i,x,y,X,x1,x2,Y,y1,y2,Ni=0,Nt=0;
	Log	*L;
	FILE	*F;
	double	c,a,c1,c2,r1,r2,d,i_d,t_d,p,p1,p2;
	Word	*Wd,**Wi,*Wt,*W1,*W2;
	FOM	**P;

	if (argc!=3 && argc!=5 && argc!=7)
	    Error(
#ifdef	XT
		  "usage : hp2xt HiPic/ {y1 y2}|{x1 y1 x2 y2} XT_format"
#else
		  "usage : hp2xp HiPic/ {y1 y2}|{x1 y1 x2 y2} XP_format"
#endif
		 );

	SearchRawFiles(argv[1],&O,&D,&N,&Q);

	(void)sprintf(S,"%s/%s",argv[1],D);
	(void)fclose(Open(S,&Nx,&Ny));

	if ((L=(Log *)malloc(sizeof(Log)*(N+1)))==NULL)
	    Error("no memory for log data.");

	for (n=0; n<=N; n++) L[n].t=0;

	(void)sprintf(S,"%s/%s",argv[1],O);
	if ((F=fopen(S,"r"))==NULL) Abort(S,"file not found.");

	while (fgets(S,LEN,F)!=NULL)
	    if (sscanf(S,"%d %lf %lf %d%n",&n,&c,&a,&t,&i)==4) {
		S[i]='\0';

		if (n<0 || n>=N || t<0 || t>1) Abort(S,"bad log data.");

		if (L[n].t!=0) Abort(S,"duplicated image number.");

		if (Q[n]==NULL) Abort(S,"missing image file.");

		(void)sprintf(S,"%s/%s",argv[1],Q[n]);
		(void)fclose(Open(S,&x,&y));

		if (Nx!=x || Ny!=y) Abort(S,"image size not match.");

		L[n].n=n; L[n].c=c; L[n].a=a; L[n].t=t*2-1;

		if (Ni+Nt==0) c1=c2=c; else if (c1>c) c1=c;
				       else if (c2<c) c2=c;

		if (t==0) ++Ni; else ++Nt;
	    }

	(void)fclose(F);

	if (Ni==0 || Nt==0) Error("no I0- and/or I-image file.");

	switch(argc) {
	    case 3: X=Nx; x1=0; x2=Nx-1;
		    Y=Ny; y1=0; y2=Ny-1; break;
	    case 5: X=Nx; x1=0; x2=Nx-1;
		    Y=1-(y1=atoi(argv[2]))+(y2=atoi(argv[3])); break;
	    case 7: X=1-(x1=atoi(argv[2]))+(x2=atoi(argv[4]));
		    Y=1-(y1=atoi(argv[3]))+(y2=atoi(argv[5])); break;
	}
	if (x1<0 || x2>=Nx || X<=0 ||
	    y1<0 || y2>=Ny || Y<=0) Error("bad image area.");

	if ((Wd=(Word *)malloc((size_t)2*Y*X))==NULL ||
	    (Wi=(Word **)malloc(sizeof(Word *)*Ni))==NULL ||
	    (*Wi=(Word *)malloc((size_t)Ni*2*Y*X))==NULL ||
	    (Wt=(Word *)malloc((size_t)2*Y*X))==NULL ||
	    (P=(FOM **)malloc(sizeof(FOM *)*Y))==NULL ||
	    (*P=(FOM *)malloc(sizeof(FOM)*Y*X))==NULL)
	    Error("no memory for image data.");

	(void)sprintf(S,"%s/%s",argv[1],D);
	Read(S,x1,y1,x2,y2,Wd);

	qsort(L,N+1,sizeof(Log),(int (*)())Compare);

	for (n=0; n<Ni; n++) {
	    if (n>0) Wi[n]=Wi[n-1]+(size_t)Y*X;

	    (void)sprintf(S,"%s/%s",argv[1],Q[L[n].n]);
	    Read(S,x1,y1,x2,y2,Wi[n]);
	}
	if (Ni==1) {
	    Ni=2; L[1].n=L[0].n; L[0].c=c1; L[1].c=c2;
	}
	for (y=1; y<Y; y++) P[y]=P[y-1]+X;

	for (t=0, n=N+1-Nt; n<=N; n++, t++) {
	    (void)sprintf(S,"%s/%s",argv[1],Q[L[n].n]);
	    Read(S,x1,y1,x2,y2,Wt);

	    c=L[n].c; for(i=Ni-2; i>0; i--) if (c>=L[i].c) break;

	    W1=Wi[i]; W2=Wi[i+1]; r1=1.0-(r2=(c-L[i].c)/(L[i+1].c-L[i].c));

	    for (i=y=0; y<Y; y++)
	    for (x=0; x<X; x++, i++) {
		d=(double)Wd[i];
		P[y][x]=p=((i_d=r1*(double)W1[i]+r2*(double)W2[i]-d)>0.0 &&
			   (t_d=(double)Wt[i]-d)>0.0)?LOG(i_d,t_d):0.0;

		if (i==0) p1=p2=p; else if (p1>p) p1=p; else if (p2<p) p2=p;
	    }
	    (void)printf("%d\t%lg\t%lf\t%lf\n",t,L[n].a,p1,p2);

	    (void)sprintf(S,argv[argc-1],t);
	    StoreImageFile_Raw(S,X,Y,p1,p2,P);
	}
	free(*P); free(P); free(Wt); free(*Wi); free(Wi); free(Wd); free(L);

	for (n=N-1; n>=0; n--) if (Q[n]!=NULL) free(Q[n]);

	free(Q); free(D); free(O);

	return 0;
}
