#!/bin/csh
# usage : plot_[a,p].csh HG.csv {CTV1 CTV2} {HG_size_inch}
#
set inch=3
set pt=(12 18 24)
#
alias STDERR 'echo echo 1\>\&2 "\!*" | /bin/sh'
#
set csh=`basename $0`
switch($csh)
  case plot_a.csh :
    set ctv='CT-value (estimated LAC, 1/cm)' ; breaksw
  case plot_p.csh :
    set ctv='CT-value (estimated RID, 1e-6)' ; breaksw
  default :
    STDERR $csh : unknown C-shell script. ; exit 1
endsw
#
if ($#argv == 0 || $#argv > 4) then
  STDERR usage : $csh HG.csv \{CTV1 CTV2\} \{HG_size_inch\} ; exit 1
endif
#
set csv=$argv[1]
if (!(-e $csv)) then
  STDERR $argv[1] : CSV file not found. ; exit 1
endif
#
if ($#argv == 1 || $#argv == 2) then
  set x=(`awk -F, 'NF==4 { print ($2+$3)/2 }' $csv | \
          sort -n | sed -n -e 1p -e \$p`)
else
  set x=(`echo "$argv[2] ; $argv[3]" | bc -l`)
  if (`echo $x | awk '{ print (NF!=2 || $1>=$2) }'`) then
    STDERR "$argv[2]" \~ "$argv[3]" : bad CT-value range. ; exit 1
  endif
endif
#
if ($#argv == 1 || $#argv == 3) then
  set size=$inch
  set font=($pt)
else
  set size=`echo "$argv[$#argv]" | bc -l`
  if (`echo $size | awk '{ print (NF!=1 || $1<=0) }'`) then
    STDERR $argv[$#argv] : bad histogram size in inch. ; exit 1
  endif
  set font=(`echo "r=${size}/$inch ; r*$pt[1] ; r*$pt[2] ; r*$pt[3]" | bc -l`)
endif
#
set y=(`awk -F, -v x1=$x[1] -v x2=$x[2] \
            'NF==4 { if ((x=($2+$3)/2)>=x1 && x<=x2) { \
                       if (y1=="" || $4<y1) y1=$4; \
                       if (y2=="" || $4>y2) y2=$4 }} \
             END { printf (y1>=10^10)?"%g\n":"%f\n",y1; \
                   printf (y2>=10^10)?"%g\n":"%f\n",y2 }' $csv`)
set z=(`awk -F, -v x1=$x[1] -v x2=$x[2] \
            'NF==4 { ++S; Sx+=(x=($2+$3)/2); Sxx+=(x*x); \
                          Sy+=$1; Syy+=($1*$1); Sxy+=(x*$1) } \
             END { a=S*Sxy-Sx*Sy; b=Sxx*Sy-Sx*Sxy; c=S*Sxx-Sx*Sx; \
                   printf "%f\n%f\n",(x1*a+b)/c,(x2*a+b)/c }' $csv`)
#
set label=`echo $csv | sed -e s/\\.csv\$//g -e s@//@\\\\n@g`
#
cat <<fin | gnuplot
  set terminal postscript eps size ${size},$size font 'Helvetica,${font[2]}'
  unset key
  set grid
  set logscale y
  set xrange [${x[1]}:${x[2]}] 
  set yrange [${y[1]}:${y[2]}]
  set x2range [${z[1]}:${z[2]}]
  set xtics font 'Helvetica,${font[1]}'
  set ytics rotate by 90 font 'Helvetica,${font[1]}'
  set x2tics font 'Helvetica,${font[1]}'
  set xlabel '${ctv}' offset 0,-0.25 font 'Helvetica,${font[2]}'
  set ylabel 'log(pixels)' offset 0.25,0 font 'Helvetica,${font[2]}'
  set label "${label}" at graph 0.9,0.85 right font 'Helvetica,${font[3]}'
  set datafile separator ","
  plot "${csv}" using ((\$2+\$3)/2):4 with lines linewidth 2
fin
