
#include <stdio.h>
#include <stdlib.h>
#include "csi_f.h"
#include "vlr.h"
#include "rif_r.h"
#include "sif_r.h"

#ifdef	DPC
#include "rr2dpc.h"

#define EPS	1e-9
#else
#include "ri2xp.h"
#endif

extern void	Error(char *msg);

#define LEN	2048

int	main(int argc,char **argv)
{
	char	**paths,path[LEN];
	int	Nx,Ny,Nz,z1,z2,Nv,z,y1,y2,Nl,v,l,nx,ny,y,x;
	VLR	*vlr;
	FOM	***sg,**sg_l,**fom,*fom_y,*sg_lv,min,max;
#ifdef	DPC
	double	sdd,Dp,Dr,sign;

	if (argc!=7 && argc!=8 && argc!=9 && argc!=10)
	    Error(
"usage : rr2sg RR/ nameFile VLR_file {layer1 layer2} SDD {-}Dp {Dr} SG_format"
		 );
#else
	if (argc!=5 && argc!=7)
	Error("usage : ri2sg RI/ nameFile VLR_file {layer1 layer2} SG_format");
#endif
	paths=CheckSliceImages_Float(argv[1],argv[2],&Nx,&Ny,&Nz,NULL);

	vlr=ViewRangeListWithLeftAndRightMargin(Nz,argv[3]);

	for (z1=0   ; z1<Nz; z1++) if (vlr[z1].view) break;
	for (z2=Nz-1; z2>=0; z2--) if (vlr[z2].view) break;

	if ((Nv=z2-z1+1)<=0) Error("no view exists.");

	for (z=z1; z<=z2; z++) if (!vlr[z].view) Error("gap of view exists.");
#ifdef	DPC
	if (argc==7 || argc==8)
#else
	if (argc==5)
#endif
	{
	    y1=0; y2=Ny-1;
	}
	else
	    if ((y1=atoi(argv[4]))<0 ||
		(y2=atoi(argv[5]))>=Ny || y1>y2) Error("bad layer range.");

	Nl=y2-y1+1;

	if ((sg=(FOM ***)malloc(sizeof(FOM **)*Nl))==NULL ||
	    (*sg=(FOM **)malloc(sizeof(FOM *)*Nl*Nv))==NULL ||
	    (**sg=(FOM *)malloc(sizeof(FOM)*Nl*Nv*Nx))==NULL)
	    Error("memory allocation error.");

	sg_l=sg[0]; for (v=1; v<Nv; v++) sg_l[v]=sg_l[v-1]+Nx;

	for (l=1; l<Nl; l++) {
	    sg_l=sg[l]=sg[l-1]+Nv; for (v=0; v<Nv; v++) sg_l[v]=sg_l[v-1]+Nx;
	}
#ifdef	DPC
	if (argc==7 || argc==8) {
	    sdd=atof(argv[4]);
	    Dp=atof(argv[5]);
	}
	else {
	    sdd=atof(argv[6]);
	    Dp=atof(argv[7]);
	}
	if (sdd<EPS) Error("bad sample-detector distance.");

	if (Dp>=(-EPS) && Dp<=EPS)
	    Error("bad interval of detectors on refraction profile.");

	if (argc==8 || argc==10) {
	    if ((Dr=atof(argv[argc-2]))<EPS)
		Error("bad interval of detectors.");

	    sign=Dp/sdd*Dr*2.0;
	}
	else
	    sign=Dp/sdd;
#endif
	for (v=z=0; z<Nz; z++) {
	    if (vlr[z].view) {
		ReadImageFile_Raw(paths[z],&nx,&ny,&fom);
#ifdef  DPC
		RR2DPC(nx,ny,fom,vlr[z].left,vlr[z].right,sign);
#else
		RI2XP(nx,ny,fom,vlr[z].left,vlr[z].right);
#endif
		for (l=y=0; y<ny; y++) {
		    fom_y=fom[y];
		    if (y1<=y && y<=y2) {
			sg_lv=sg[l++][v];
#ifdef	DPC
			if (argc==8 || argc==10) {
/*
			    if (nx&1)
				sg_lv[x=nx-1]=0.0;
			    else
*/
			    sg_lv[x=nx-2]=(-fom_y[nx-1]);
			    while ((x-=2)>=0) sg_lv[x]=sg_lv[x+2]-fom_y[x+1];
/**/
			    if (nx&1)
				sg_lv[x=0]=0.0;
			    else
/**/
			    sg_lv[x=1]=fom_y[0];
			    while ((x+=2)<nx) sg_lv[x]=sg_lv[x-2]+fom_y[x-1];

			    x=nx;
			}
			else
#endif
			for (x=0; x<nx; x++) sg_lv[x]=fom_y[x];

			for (; x<Nx; x++) sg_lv[x]=0.0;
		    }
		    free(fom_y);
		}
		free(fom);

		for (; y<Ny; y++)
		    if (y1<=y && y<=y2) {
			sg_lv=sg[l++][v]; for (x=0; x<Nx; x++) sg_lv[x]=0.0;
		    }

		++v;
	    }
	    free(paths[z]);
	}
	free(vlr); free(paths);

	for (y=y1, l=0; l<Nl; l++, y++) {
	    min=max=(sg_l=sg[l])[0][0];
	    for (v=0; v<Nv; v++) {
		sg_lv=sg_l[v];
		for (x=0; x<Nx; x++)
		    if (min>sg_lv[x]) min=sg_lv[x]; else
		    if (max<sg_lv[x]) max=sg_lv[x];
	    }
	    (void)printf("%d\t%lf\t%lf\n",y,(double)min,(double)max);

	    (void)sprintf(path,argv[argc-1],y);
	    StoreImageFile_Raw(path,Nx,Nv,(double)min,(double)max,sg_l);
	}
	free(**sg); free(*sg); free(sg);

	return 0;
}
