
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "csi_f.h"
#include "vlr.h"
#include "rif_r.h"
#include "sif_r.h"

#ifdef	DPC
#include "rr2dpc.h"

#define EPS	1e-9
#else
#include "ri2xp.h"
#endif

extern void	Error(char *msg);

int	main(int argc,char **argv)
{
	char	**paths;
	int	Nx,Ny,Nz,z1,z2,Nv,z,y,v,nx,ny,x;
	VLR	*vlr;
	FOM	**sp,**fom;
	double	sum,min,max,mom,d,Dr=1.0;
#ifdef	DPC
	double	*line,sdd,Dp,sign;

	if (argc!=7 && argc!=8)
	    Error("usage : rr2sp RR/ nameFile VLR_file SDD {-}Dp {Dr} SP.tif");
#else
	if (argc!=5 && argc!=6)
	    Error("usage : ri2sp RI/ nameFile VLR_file {Dr} SP.tif");
#endif
	paths=CheckSliceImages_Float(argv[1],argv[2],&Nx,&Ny,&Nz,NULL);

	vlr=ViewRangeListWithLeftAndRightMargin(Nz,argv[3]);

	for (z1=0   ; z1<Nz; z1++) if (vlr[z1].view) break;
	for (z2=Nz-1; z2>=0; z2--) if (vlr[z2].view) break;

	if ((Nv=z2-z1+1)<=0) Error("no view exists.");

	for (z=z1; z<=z2; z++) if (!vlr[z].view) Error("gap of view exists.");

	if (
#ifdef	DPC
	    (line=(double *)malloc(sizeof(double)*Nx))==NULL ||
#endif
	    (sp=(FOM **)malloc(sizeof(FOM *)*Ny))==NULL ||
            (*sp=(FOM *)malloc(sizeof(FOM)*Ny*Nv))==NULL)
	    Error("memory allocation error.");

	for (y=1; y<Ny; y++) sp[y]=sp[y-1]+Nv;
#ifdef	DPC
	if ((sdd=atof(argv[4]))<EPS) Error("bad sample-detector distance.");

	if (fabs(Dp=atof(argv[5]))<=EPS)
	    Error("bad interval of detectors on refraction profile.");

	sign=2.0*Dp/sdd;

	if (argc==8) Dr=atof(argv[6]);
#else
	if (argc==6) Dr=atof(argv[4]);
#endif
	for (v=z=0; z<Nz; z++) {
	    if (vlr[z].view) {
		ReadImageFile_Raw(paths[z],&nx,&ny,&fom);
#ifdef	DPC
		RR2DPC(nx,ny,fom,vlr[z].left,vlr[z].right,sign);
#else
		RI2XP(nx,ny,fom,vlr[z].left,vlr[z].right);
#endif
		for (y=0; y<ny; y++) {
		    sum=0.0;
#ifdef	DPC
/*
		    if (nx&1) line[x=nx-1]=0.0; else
*/
		    line[x=nx-2]=(-fom[y][nx-1]);
		    while ((x-=2)>=0) line[x]=line[x+2]-fom[y][x+1];
/**/
		    if (nx&1) line[x=0]=0.0; else
/**/
		    line[x=1]=fom[y][0];
		    while ((x+=2)<nx) line[x]=line[x-2]+fom[y][x-1];

		    for (x=0; x<nx; x++) sum+=line[x];
#else
		    for (x=0; x<nx; x++) sum+=fom[y][x];
#endif
		    sp[y][v]=sum*Dr; free(fom[y]);
		}
		for (; y<Ny; y++) sp[y][v]=0.0;

		++v; free(fom);
	    }
	    free(paths[z]);
	}
	free(vlr); free(paths);
#ifdef	DPC
	free(line);
#endif
	min=max=sp[0][0];
	for (y=0; y<Ny; y++) {
	    sum=mom=0.0;
	    for (v=0; v<Nv; v++) {
		sum+=(d=sp[y][v]); mom+=(d*d); if (min>d) min=d; else
					       if (max<d) max=d;
	    }
	    sum/=(double)Nv;
	    (void)printf("%d\t%lf\t%lf\n",y,sum,sqrt(mom/(double)Nv-sum*sum));
	}
	StoreImageFile_Raw(argv[argc-1],Nv,Ny,min,max,sp);

	free(*sp); free(sp);

	return 0;
}
