
#include <stdio.h>
#include <stdlib.h>
#include "csi_f.h"
#include "vlr.h"
#include "rif_r.h"
#include "sgif.h"

#ifdef	DPC
#include "rr2dpc.h"

#define EPS	1e-9
#else
#include "ri2xp.h"
#endif

extern void	Error(char *msg);

#define BYTE	unsigned char

int	main(int argc,char **argv)
{
	char	**paths;
	int	x,y,Z,Bx,By,z,Ox,Oy,Nx,Ny,ny,nx,oy,by,ox,bx;
	VLR	*vlr;
	double	min,max,unit,bias,d;
	FILE	*file;
	FOM	**fom;
	BYTE	*byte;
#ifdef	DPC
	double	sdd,Dp,sign;

#define NO_MMG	8
#else
#define NO_MMG	6
#endif
	if (argc!=NO_MMG && argc!=NO_MMG+1 && argc!=NO_MMG+3)
	    Error(
#ifdef	DPC
"usage : rr2dpc_gif RR/ nameFile VLR_file Bx By SDD {-}Dp {{min max} DPC.gif}"
#else
#ifdef	XT
	    "usage : ri2xt_gif RI/ nameFile VLR_file Bx By {{min max} XT.gif}"
#else
	    "usage : ri2xp_gif RI/ nameFile VLR_file Bx By {{min max} XP.gif}"
#endif
#endif
		 );

	paths=CheckSliceImages_Float(argv[1],argv[2],&x,&y,&Z,NULL);

	vlr=ViewRangeListWithLeftAndRightMargin(Z,argv[3]);

	if ((Bx=atoi(argv[4]))<1 ||
	    (By=atoi(argv[5]))<1) Error("bad block size.");

	if (argc==NO_MMG+3) {
	    min=atof(argv[NO_MMG]);
	    if ((max=atof(argv[NO_MMG+1]))<=min) Error("bad value range.");

	    unit=255.0/(max-min); bias=0.5-min*unit;
	}
	if (argc!=NO_MMG) {
	    if (argv[argc-1][0]=='-')
		file=stdout;
	    else
	    if ((file=fopen(argv[argc-1],"wb"))==NULL) Error("file not open.");

	    InitStoreGIF(file,(x+Bx-1)/Bx,(y+By-1)/By,
			      StoreGIF_Gray,StoreGIF_Gray,StoreGIF_Gray);
	}
#ifdef	DPC
	if ((sdd=atof(argv[6]))<EPS) Error("bad sample-detector distance.");

	if ((Dp=atof(argv[7]))>=(-EPS) && Dp<=EPS)
	    Error("bad interval of detectors on refraction profile.");

	sign=Dp/sdd;
#endif
	for (z=0; z<Z; z++) {
	    if (vlr[z].view) {
		ReadImageFile_Raw(paths[z],&Ox,&Oy,&fom);
#ifdef	DPC
		RR2DPC(Ox,Oy,fom,vlr[z].left,vlr[z].right,sign);
#else
		RI2XP(Ox,Oy,fom,vlr[z].left,vlr[z].right);
#endif
		Nx=(Ox+Bx-1)/Bx;
		Ny=(Oy+By-1)/By;
		for (y=ny=0; ny<Ny; ny++, y+=By)
		for (x=nx=0; nx<Nx; nx++, x+=Bx) {
		    d=0.0;
		    for (oy=y, by=0; oy<Oy && by<By; oy++, by++)
		    for (ox=x, bx=0; ox<Ox && bx<Bx; ox++, bx++) d+=fom[oy][ox];

		    fom[ny][nx]=(d/=((double)bx*(double)by));

		    if (argc!=NO_MMG+3)
			if (nx==0 && ny==0)
			    min=max=d;
			else
			    if (min>d) min=d; else if (max<d) max=d;
		}
		if (argc==NO_MMG)
		    (void)printf("%d\t%lf\t%lf\n",z,min,max);
		else {
		    if (argc==NO_MMG+1) {
			unit=(max<=min)?1.0:255.0/(max-min); bias=0.5-min*unit;
		    }
		    for (y=0; y<Ny; y++) {
			byte=(BYTE *)fom[y];
			for (x=0; x<Nx; x++)
			    byte[x]=(fom[y][x]<min)?0:
				    (fom[y][x]>max)?255:
				    (BYTE)(fom[y][x]*unit+bias);
		    }
		    StoreGIF(file,0,0,Nx,Ny,NULL,NULL,NULL,(BYTE **)fom);
		}
		for (y=Oy-1; y>=0; y--) free(fom[y]); free(fom);
	    }
	    free(paths[z]);
	}
	if (argc!=NO_MMG) {
	    TermStoreGIF(file); if (file!=stdout) (void)fclose(file);
	}
	free(vlr); free(paths);

	return 0;
}
