
#include <stdio.h>
#include <stdlib.h>
#include "csi_f.h"
#include "vlr.h"
#include "rif_r.h"
#include "sif_r.h"

#ifdef	DPC
#include "rr2dpc.h"

#define EPS	1e-9
#else
#include "ri2xp.h"
#endif

extern void	Error(char *msg);

#define LEN	2048

int	main(int argc,char **argv)
{
	char	**paths,path[LEN];
	int	Z,Bx,By,name,z,Ox,Oy,Nx,Ny,y,ny,x,nx,oy,by,ox,bx;
	VLR	*vlr;
	FOM	**fom;
	double	sum,min,max;
#ifdef	DPC
	double	sdd,Dp,sign;

	if (argc!=9)
	    Error("usage : rr2dpc RR/ nameFile VLR_file Bx By SDD {-}Dp DPC/");
#else
	if (argc!=7)
	    Error(
#ifdef	XT
		  "usage : ri2xt RI/ nameFile VLR_file Bx By XT/"
#else
		  "usage : ri2xp RI/ nameFile VLR_file Bx By XP/"
#endif
		 );
#endif
	paths=CheckSliceImages_Float(argv[1],argv[2],NULL,NULL,&Z,NULL);

	vlr=ViewRangeListWithLeftAndRightMargin(Z,argv[3]);

	if ((Bx=atoi(argv[4]))<1 ||
	    (By=atoi(argv[5]))<1) Error("bad block size.");

	name=0; while (argv[1][name++]!='\0') ;

#ifdef	DPC
	if ((sdd=atof(argv[6]))<EPS) Error("bad sample-detector distance.");

	if ((Dp=atof(argv[7]))>=(-EPS) && Dp<=EPS)
	    Error("bad interval of detectors on refraction profile.");

	sign=Dp/sdd;
#endif
	for (z=0; z<Z; z++) {
	    if (vlr[z].view) {
		ReadImageFile_Raw(paths[z],&Ox,&Oy,&fom);
#ifdef	DPC
		RR2DPC(Ox,Oy,fom,vlr[z].left,vlr[z].right,sign);
#else
		RI2XP(Ox,Oy,fom,vlr[z].left,vlr[z].right);
#endif
		Nx=(Ox+Bx-1)/Bx;
		Ny=(Oy+By-1)/By;
		for (y=ny=0; ny<Ny; ny++, y+=By)
		for (x=nx=0; nx<Nx; nx++, x+=Bx) {
		    sum=0.0;
		    for (oy=y, by=0; oy<Oy && by<By; oy++, by++)
		    for (ox=x, bx=0; ox<Ox && bx<Bx; ox++, bx++)
			sum+=fom[oy][ox];

		    fom[ny][nx]=sum/((double)bx*(double)by);
		}
		min=max=fom[0][0];
		for (y=0; y<Ny; y++)
		for (x=0; x<Nx; x++)
		    if (min>fom[y][x]) min=fom[y][x]; else
		    if (max<fom[y][x]) max=fom[y][x];

		(void)printf("%d\t%lf\t%lf\n",z,min,max);

		(void)sprintf(path,"%s/%s",argv[argc-1],paths[z]+name);
		StoreImageFile_Raw(path,Nx,Ny,min,max,fom);

		for (y=Oy-1; y>=0; y--) free(fom[y]); free(fom);
	    }
	    free(paths[z]);
	}
	free(vlr); free(paths);

	return 0;
}
