
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rif_f.h"

extern void	Error(char *msg);

static int	Nx,Ny;

static FOM	**ReadFloatTIFF(char *path,int first)
{
	FOM	**fom;
	char	*id,*u_c,
		*using="using integer PV as is floating-point PV",
		*converting="converting PV by TIFF image description tag";
	int	x,y,
		bps=ReadImageFile_Float(path,&x,&y,&fom,&id);
	double	v0,v1,dv;

	if (first) {
	    Nx=x; Ny=y;
	}
	else
	    if (Nx!=x || Ny!=y) Error("image size not match.");

	if (bps!=0) {
	    if (id==NULL || sscanf(id,"%lf %lf",&v0,&v1)!=2 || v0>=v1)
		u_c=using;
	    else {
		dv=(v1-v0)/(double)((1ULL<<bps)-1);
		for (y=0; y<Ny; y++)
		for (x=0; x<Nx; x++) fom[y][x]=v0+dv*fom[y][x];

		u_c=converting;
	    }
	    (void)fprintf(stderr,"%s : %s (warning).\n",path,u_c);
	}
	if (id!=NULL) free(id);

	return fom;
}

#define	PI	3.14159265358979323846

typedef struct {
		double	r,i;
	} Complex;

static int	SetUpFFT(int N,Complex **E,Complex **C)
{
	int	L,l;
	Complex	*e;
	double	d;

	for (L=1; L<N; L<<=1) ;

	if ((*E=(Complex *)malloc(sizeof(Complex)*L))==NULL ||
	    (*C=(Complex *)malloc(sizeof(Complex)*L))==NULL) return 0;

	for (e=(*E), l=0; l<L; l++, e++) {
	    d=PI*(double)l/(double)L; e->r=cos(d); e->i=sin(d);
	}
	return L;
}

#define SIGN	(-1)

static void	FastFourierTransform(int L,Complex *E,Complex *C)
{
	int	j,k,m,n,p,q,L2=L>>1;
	Complex	c,*e,*a,*b;
	double	r,i,d;

	for (j=L-1, k=L-2; k>0; k--) {
	    for (m=L2; (j^=m)&m; m>>=1) ;

	    if (j>k) {
		c=C[j]; C[j]=C[k]; C[k]=c;
	    }
	}
	for (j=2, k=1, m=L2, n=L; n>1; n=m, m>>=1, k=j, j<<=1)
	    for (e=E, p=0; p<k; p++, e+=n) {
		r=e->r; i=e->i*(double)SIGN;
		for (a=C+p, q=0; q<m; q++, a+=j) {
		    c=(*(b=a+k)); b->r=a->r-(d=c.r*r-c.i*i); a->r+=d;
				  b->i=a->i-(d=c.r*i+c.i*r); a->i+=d;
		}
	    }
}

#define HZ_IS_GIVEN	(argv[1][0]=='-')

int	main(int argc,char **argv)
{
	FOM	**M,**T;
	int	y1,y2,y,x,N,X,L,H,n;
	double	df,dt,t1,t2,t,T1,T2,*S,m;
	Complex	*E,*C;

	if (argc!=3 && argc!=5)
	    Error("usage : tm2psd -Hz|time.tif mean.tif {view1 view2}");

	M=ReadFloatTIFF(argv[2],1);

	if (argc==3) {
	    y1=0; y2=Ny-1;
	}
	else
	    if ((y1=atoi(argv[3]))<0 ||
		(y2=atoi(argv[4]))>=Ny || y1>y2) Error("bad view range");

	if (HZ_IS_GIVEN) {
	    if ((df=atof(argv[1]+1))<=0.0) Error("bad sampling frequency.");

	    dt=1.0/df; N=Nx;
	}
	else {
	    T=ReadFloatTIFF(argv[1],0);

	    for (y=y1; y<=y2; y++) {
		t1=t2=T[y][0]; T[y][0]=0.0;
		for (x=1; x<Nx; x++) {
		    if (t2>=(t=T[y][x])) Error("bad time sequence.");

		    T[y][x]-=t1; t2=t;
		}
		t=t2-t1; if (y==y1) T1=T2=t; else if (T1>t) T1=t; else
						  if (T2<t) T2=t;
	    }
	    df=1.0/(dt=T1/(double)(X=Nx-1)); N=(int)floor(T2/dt)+1;
	}
	if ((L=SetUpFFT(N,&E,&C))==0 ||
	    (S=(double *)malloc(sizeof(double)*(H=L/2)))==NULL)
	    Error("memory allocation error.");

	for (n=0; n<H; n++) S[n]=0.0;

	for (y=y1; y<=y2; y++) {
	    m=0.0; for (x=0; x<Nx; x++) m+=M[y][x]; m/=(double)Nx;

	    if (HZ_IS_GIVEN)
		for (n=0; n<N; n++) {
		    C[n].r=M[y][n]-m; C[n].i=0.0;
		}
	    else {
		N=(int)floor(T[y][X]/dt); x=1;
		for (t=fmod(T[y][X],dt)/2.0, n=0; n<=N; n++, t+=dt) {
		    while (t>T[y][x]) ++x;

		    t1=t-T[y][x-1]; t2=T[y][x]-t;
		    C[n].r=(t1*M[y][x]+t2*M[y][x-1])/(t1+t2)-m; C[n].i=0.0;
		}
	    }
	    for (; n<L; n++) C[n].r=C[n].i=0.0;

	    FastFourierTransform(L,E,C);

	    S[0]+=m; for (n=1; n<H; n++) S[n]+=(C[n].r*C[n].r+C[n].i*C[n].i);
	}
	(void)printf("%lg\t%lg\n",df,
				  S[0]/(double)(y2-y1+1));
	for (n=1; n<H; n++)
	    (void)printf("%le\t%le\n",df*(double)n/(double)L,
				      dt*S[n]/(double)(y2-y1+1)/(double)L);

	free(S); free(C); free(E);

	if (!HZ_IS_GIVEN) {
	    for (y=0; y<Ny; y++) free(T[y]); free(T);
	}
	for (y=0; y<Ny; y++) free(M[y]); free(M);

	return 0;
}
